//Copyright 2011 INKA - HTW Berlin
//
//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
//  Created by Maurus Rohrer maurus.rohrer@gmail.com

//TODO: changeAntenna function

#include "MetraTecDeviceManager.h"
#include "SerialDevice.c"
#include <algorithm>
#include "MifareDesfire.h"


MetraTecDeviceManager::MetraTecDeviceManager(std::string dev): device(dev){
    //constructor of MetraTecDeviceManager
}

bool MetraTecDeviceManager::connect(){
    fileDescriptor = OpenSerialDevice(device.c_str());
    
    //no file desc
    if (fileDescriptor <0) return false;
    else return true;
}

bool MetraTecDeviceManager::disconnect(){
    CloseSerialDevice(fileDescriptor);
    return true;
}

std::vector<Transponder> MetraTecDeviceManager::readTransponders(){
    std::vector<Transponder> transponders;
    std::vector<std::string> ids;
    //get all Transponder in range
    std::string result = WriteCommand(fileDescriptor, "INV\r");
    splitString(result, &ids);
    // remove non ids
    ids.erase( std::remove_if( ids.begin(), ids.end(), is_id() ), ids.end() );
    //create Transponder Object for each id
    std::vector<std::string>::iterator it; 
    for(it = ids.begin(); it < ids.end(); it++){
        switch (active_type) {
            case ICODE:{
                ICodeTransponder ict(*it);
                transponders.push_back(ict);
                break;
            }
            default:{
                Transponder t(*it);
                transponders.push_back(t);
                break;
            }
        }
    }
    return transponders;
}

bool MetraTecDeviceManager::isNDEFConform(Transponder* transponder){
    //get data of block 0
    std::string answer = readDataOnBlockOnTranspond(transponder->getID(), 0);
    std::transform(answer.begin(), answer.end(),answer.begin(), ::tolower);
    
    //block 0 should contain CC (Capability Container)
    if(answer != "e1400e01") return false;

    //get block 1
    answer = readDataOnBlockOnTranspond(transponder->getID(), 1);
    std::transform(answer.begin(), answer.end(),answer.begin(), ::tolower);
    
    //first byte should be 03h
    std::string tlvType = "03";
    if (answer.compare(0, tlvType.length(), tlvType) != 0) return false;
    
    //now its ndef conform
    return true;
}

bool MetraTecDeviceManager::makeNDEFConform(Transponder* transponder){   
    //wirte CC on block 0
    if (!writeDataOnBlockOnTranspond(transponder->getID(), "e1400e01", 0)) return false;
    //write TLV Tag (03) and no NDEF message(00) and Terminator TLV (fe)
    if (!writeDataOnBlockOnTranspond(transponder->getID(), "0300fe00", 1)) return false;
    //succeded
    return true;
}


bool MetraTecDeviceManager::writeNDEFMessage(Transponder* transponder ,NDEFMessage* ndefMessage){
    //TODO: tlv should be hex string!!!!!! change method if l and v are normal char strings
    
    //write NDEF Message (TLV) on Transponder
    std::string tlv = ndefMessage->getTLV();
    unsigned long sizeOfTlv = tlv.length();
    //NDEF Message to big for Transponder
    if(sizeOfTlv*2>108) {
        //TODO: error message
        std::cout << "content to big for tranpsonder" <<std::endl;
        return false;
    }
    
    //append empty byte, metraTec only wirtes compelete blocks
    while (tlv.length()%8!=0) {
        tlv.append("00");
    }
    //get block to write
    int blocksToWrite = (int)tlv.length()/8;
    int blockNr=1;
    
    //wirte all blocks
    do{
        if (!writeDataOnBlockOnTranspond(transponder->getID(), tlv.substr((blockNr-1)*8,8), blockNr)) return false;
        blockNr++;
        blocksToWrite--;
    }while (blocksToWrite > 0);
    return true;
}

NDEFMessage MetraTecDeviceManager::readNDEFMessage(Transponder* transponder){
    //TODO: testing on metratec reader
    //check if transponder is ndef type
    if (isNDEFConform(transponder)) {
        //get length of ndefmessage (hex representation)
        
        std::string answer = readDataOnBlockOnTranspond(transponder->getID(), 1);
        int ndefLengthHex = mifare_desfire::hexStringToInt(answer.substr(2,2));
        
        int blocksToRead;
        if (ndefLengthHex%8==0) {
            blocksToRead = (int)ndefLengthHex/8;
        }else{
            blocksToRead = (int) ndefLengthHex/8 +1;
        }
        std::string ndefMessageString = answer.substr(4,4);
        int blockNr =2;
        do{
            answer = readDataOnBlockOnTranspond(transponder->getID(), blockNr);
            if(answer.size()!=0){
                ndefMessageString.append(answer);
            }
            blockNr++;
            blocksToRead--;
        }while (blocksToRead > 0);
        return NDEFMessage::parseNDEFMessage(ndefMessageString.substr(0,ndefLengthHex));
    }
    //return empty NDEFMessage
    return NDEFMessage();
}


bool MetraTecDeviceManager::configureReader(TransponderType type){
    std::string result;
    switch (type) {
        case ICODE:
            //set local type
            active_type = type;
            //SET MOD 156
            result = WriteCommand(fileDescriptor, "MOD 156\r");
            if(result.find("OK!")==std::string::npos) {
                CloseSerialDevice(fileDescriptor);
                return false;
            }
            // Icode settings
            result = WriteCommand(fileDescriptor, "SRI SS 100 \r");
            if(result.find("OK!")==std::string::npos) {
                CloseSerialDevice(fileDescriptor);
                return false;
            }
            return true;
        default:
//            std::cout<< "error: transponder Type not suportet") << std::endl;
            return false;
    }
}

//wirte metrTec command directly to reader (TODO: testing on reader)
std::vector<std::string> MetraTecDeviceManager::writeMetraTecCommand(std::string command){
    command.append("\r");
    std::string answer = WriteCommand(fileDescriptor,command.c_str());
    std::vector<std::string> results;
    splitString(answer, &results);
    return results;
    
}

//help functions
void MetraTecDeviceManager::splitString(std::string tosplit, std::vector<std::string>* results){
    unsigned long found;
    found = tosplit.find_first_of("\\r");
    while(found != std::string::npos){
        if(found > 0){
            results->push_back(tosplit.substr(0,found));
        }
        tosplit = tosplit.substr(found+2);
        found = tosplit.find_first_of("\\r");
    }
    if(tosplit.length() > 0){
        results->push_back(tosplit);
    }
    
}

std::string MetraTecDeviceManager::byteToHexString(unsigned char c){
    std::ostringstream out;
    out << std::hex << std::setw(2) << std::setfill('0') << static_cast<unsigned int>(static_cast<unsigned char>(c));
    return out.str();
}


bool MetraTecDeviceManager::writeDataOnBlockOnTranspond(std::string id, std::string data, int blockNr){
    //create write command
    int numOfTrys=0;
    std::string command="REQ 2221";
    command.append(id);
    command.append(byteToHexString(blockNr));
    command.append(data);
    command.append(" crc\r");
    std::string answer;
    //try to write command 5 times
    do {
        if(numOfTrys>5) {
            std::cout << "Error: " << answer.substr(0,3) << std::endl;
            return false; 
        }
        answer = WriteCommand(fileDescriptor, command.c_str()); 
        numOfTrys++;
    } while ( answer.find("00")==std::string::npos);
    return true;
    
}

std::string MetraTecDeviceManager::readDataOnBlockOnTranspond(std::string id, int blockNr){
    //create read command
    std::vector<std::string> answers;
    int numOfTrys=0;
    std::string command="REQ 2220";
    command.append(id);
    command.append(byteToHexString(blockNr));
    command.append(" crc\r");
    std::string answer;
    //try to write command 5 times
    do {
        if(numOfTrys>5) {
            std::cout << "Error: " << answer.substr(0,3) << std::endl;
            return "";
        }
        answer = WriteCommand(fileDescriptor, command.c_str()); 
        numOfTrys++;
    } while ( answer.find("00")==std::string::npos);
    splitString(answer, &answers);
    answers.erase( std::remove_if( answers.begin(), answers.end(), is_data() ), answers.end() );
    //when not 1 answer something went wrong while readeing
    if(answers.size()!=1) return "";
    return answers[0].substr(2,8);
    
}


